// =====================================================================================================================
// hpomw.js
//
// Main file for the HP OMW integration service. Handles requests for HP OMW to xMatters.
//
// =====================================================================================================================
importPackage(java.lang);
importPackage(java.sql);
importPackage(java.util);
importPackage(java.io);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLMessage);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLToken);
importClass(Packages.com.alarmpoint.integrationagent.util.EventDeduplicator);
importClass(Packages.com.thoughtworks.xstream.XStream);
importClass(Packages.com.thoughtworks.xstream.converters.reflection.PureJavaReflectionProvider);

var xStream = new XStream(new PureJavaReflectionProvider())

// Core Javascript files provided by the IA as of 4.1 patch 5
load("integrationservices/hpomw/lib/javascript/core/baseclass.js");
load("integrationservices/hpomw/lib/javascript/core/logger.js");
load("integrationservices/hpomw/lib/javascript/webservices/wsutil.js");
load("integrationservices/hpomw/lib/javascript/webservices/soapfault.js");
load("integrationservices/hpomw/lib/javascript/xmatters/xmattersws.js");

// Integration-specific JS files
load("integrationservices/hpomw/hpomw-config.js");

var exceptionString = "JavaException: ";
var notePrefix = "[xMatters] - "; // The Prefix which is added to the note comments e.g. "[xMatters] "

//-----------------------------------
// Shared globals below this line
//-----------------------------------
var log = new Logger("hpomw.js: ");

//---------------------------------------------------------------------------------------
// Functions and globals below this line are used by the direct xMatters INTEGRATION
//---------------------------------------------------------------------------------------

/**
 * This is the main method for handling APXML messages sent to the
 * Integration Service by APClient.  The messages may be requests to perform
 * local activity, or they may be requests to make submissions to xMatters.
 * <p>
 * Any APXMLMessage object that this method returns will be sent to xMatters
 * via the Integration Service's outbound queues.
 */
function apia_input(apxml)
{
  var eventType = apxml.hasToken("event_type") ? apxml.getValue("event_type") : "add"
  
  if ("del".equalsIgnoreCase(eventType))
  {
    return processDelEvent(apxml);
  }
  else
  {
    return processAddEvent(apxml);
  } 
}

/**
 * Enhances the APXML for the event injection
 * @param apxml APXML Message that was received by the script
 * @return enhanced APXML Message
 */
function processAddEvent(apxml)
{
  log.info("** processAddEvent **");

  var result = null;

  // injected message fields
  var msgSrc = apxml.getValue("msg_source");

  if (msgSrc.equals("unknown0"))
  {
    apxml.setToken("msg_source", "");  // set to empty as external nodes do not have polices
    log.info("Enrichment Complete for External Node");
  } else
  {
    // strip off the version
    if (msgSrc.lastIndexOf("(") > 0)
    {
      msgSrc = msgSrc.substring(0, msgSrc.lastIndexOf("("));
    }
    apxml.setToken("msg_source", msgSrc);
    log.info("Enrichment Complete for Managed Node");
  }
  
  log.debug(" APXML to be sent to xMatters: " + apxml);

  // Forward enriched APXML to AlarmPoint, subject to deduplication
  var webService = new xMattersWS();
  var recentlyReceived = EventDeduplicator.getInstance().recentOccurrenceCount(apxml, DEDUPLICATOR_FILTER);

  if (recentlyReceived <= 1)
  {
      result = apxml;
  }
  else
  {
      log.warn("***Deduplicator Suppressed Notification ***: An event with tokens " + webService.convertTokensToString(apxml) + " has been injected into the " +
          apxml.getValue("agent_client_id") + " event domain " + recentlyReceived + " times within the configured suppression period.  It has been suppressed." );
  }

  return result;
}

/**
 * Create a new APXML Message to sent to the current event domain for the DEL
 * @param apxml APXML Message that was received by the script
 * @return APXML Message for the delete event
 */
function processDelEvent(apxml)
{
  var incidentId = apxml.getValue("incident_id");
  var request = ServiceAPI.createAPXML();
  request.setMethod("del");
  request.setSubclass("action");
  request.setToken(APXMLMessage.AGENT_CLIENT_ID, apxml.getValue(APXMLMessage.AGENT_CLIENT_ID));
  request.setToken(APXMLMessage.APIA_PROCESS_GROUP, incidentId);
  request.setToken("incident_id", incidentId);

  return request;
} 

/**
 * This is the main method for handling APXML messages sent to the
 * Integration Service by xMatters.  The messages may be responses to
 * previous submissions, or they may be requests to the Integration Service
 * that originate from xMatters
 * (e.g., via an ExternalServiceMessage/ExternalServiceRequest2).
 * <p>
 * Any APXMLMessage object that this method returns will be sent to xMatters
 * either via the Integration Service's outbound queues or directly as a
 * Web Service response, depending on the mechanism used to submit the APXML.
 */
function apia_response(apxml)
{
  var method = apxml.getMethod();
  log.info("Calling APIA_RESPONSE method = " + method);
  if (APXMLMessage.OK_METHOD.equalsIgnoreCase(method))
  {
    return handleOK(apxml);
  }
  else if (APXMLMessage.ERROR_METHOD.equalsIgnoreCase(method))
  {
    return handleERROR(apxml);
  }
  else if (APXMLMessage.SEND_METHOD.equalsIgnoreCase(method))
  {
    return handleSend(apxml);
  }
  else if (APXMLMessage.REQUEST_METHOD.equalsIgnoreCase(method))
  {
    return handleRequest(apxml);
  }
  else
  {
    throw new IllegalArgumentException("Unrecognized APXML method: " + method);
  }
}

/**
 * OK responses are returned by xMatters for each APXML message that it accepts
 * from an Integration Service.
 * <p>
 * This implementation just logs the ack.
 */
function handleOK(apxml)
{
  var id = apxml.getTransactionId();
  log.info("Submission of transaction " + id + " succeeded.");
}

/**
 * ERROR responses are returned by xMatters for each APXML message that it
 * is unable to accept from an Integration Service.
 * <p>
 * This implementation just logs the error.
 */
function handleERROR(apxml)
{
  var transaction = apxml.getTransactionId();
  var code = apxml.getValue(APXMLMessage.ERROR_CODE);
  log.error("Submission of transaction " + transaction + " failed with error code " + code);
}

/**
 * Requests are forwarded to the Send handler.
 * <p>
 * Redirects to the handleSend as messages may come from ESM or ESR to the IA.
 */
function handleRequest(apxml)
{
    log.info( "handleRequest received apxml, forwarding to handleSend" );
    return handleSend(apxml);
}

/**
 * The main routine for handling Requests starts here.
 * <p>
 * Throw an exception that will be logged by the Integration Agent.
 */
function handleSend(apxml)
{

  log.info( "Enter - handleSend received apxml: " + apxml );
  var result = null;

  // The presence of the message token means this message is a callout annotation.
  var message = apxml.getValue("message");
  var incidentId = apxml.getToken("incident_id").getValue();

  if ( message != null && incidentId != null)
  {
      result = handleCalloutAnnotate(incidentId, message, apxml);
  }
  else
  {
      result = handleApsResponses( apxml );
  }

  log.info( "Exit - handleSend");
  return result;
}


/**
 * -----------------------------------------------------------------------------
 * handleApsResponses
 *
 * Handling of delivery notifications and user responses from xMatters.
 * Decide what to do with the APXML message that has been received.
 * -----------------------------------------------------------------------------
 */
function handleApsResponses(apxml)
{
    log.info( "Enter - handleApsResponses with APXML: " + apxml);

    // Initialize xMattersWS interface building a reply to xMatters.
    var xmWebService = new xMattersWS();
    var responseToXM = xmWebService.initializeResponse(apxml);
    var responseTokens = getTokens(apxml);

    try
    {
        log.debug( "handleApsResponses - Token keys from xMatters: " + responseTokens);

        var responseAction = responseTokens.get("responseAction");
        var annotation = responseTokens.get("annotation");

        // See if the APXML message is invalid, or if we don't have anything to do
        // in response to it.
        if (!checkAPXMLTokens(responseTokens))
        {
            // Nothing to do in response to this message, so just return OK
            responseToXM.setToken(xmWebService.TOKEN_NAME, "OK", APXMLToken.Type.STRING);
        }
        else
        {
            if (responseTokens.get("responseEvent") == "SUCCESSFUL_DELIVERY"
                || responseTokens.get("responseEvent") == "DELIVERY_FAILURE")
            {
                log.debug("Received delivery notification [" + responseTokens.get("responseEvent") + "]");
                // Create an annotation for the successful or failed delivery of a notification.
                // Set the responseAction token to Annotate to pass control to the Annotate handler.
                responseAction = "Annotate";
                annotation = responseTokens.get("responseEvent") + " to " + responseTokens.get("responder");
            }
            else if (responseTokens.get("responseEvent") == "RECEIVED_RESPONSE")
            {
              if (annotation == null || annotation == "") {
                annotation = responseTokens.get("responseEvent") + " " + responseTokens.get("responseAction") + " by " + responseTokens.get("responder");
              } else {
                annotation = annotation.replace('"',"") + " by " + responseTokens.get("responder");
              }
            }

            handleResponseAction(responseAction, annotation, responseTokens, responseToXM);

            responseToXM.setToken(xmWebService.TOKEN_NAME, "OK", APXMLToken.Type.STRING);
        }
    }
    catch (e)
    {
        var errMsg = "Caught exception processing [" + responseTokens.get("responseEvent") + "|" + responseAction + "]: " + e;
        log.error(errMsg);
        responseToXM.setToken(xmWebService.TOKEN_NAME_EXCEPTION, errMsg.replace(exceptionString, ""), APXMLToken.Type.STRING);
    }
    
    log.info( "Exit - handleApsResponses");
    return responseToXM;
}

/**
 * -----------------------------------------------------------------------------
 * handleResponseAction
 *
 * Dispatch the user response to the appropriate handler function. The caller
 * may have over-ridden the responseAction and annotation tokens from the APXML
 * message, but the other tokens will be untouched.
 * -----------------------------------------------------------------------------
 */
function handleResponseAction(responseAction, annotation, responseTokens, responseToXM)
{
    log.info("Enter - handleResponseAction. Handling responseAction [" + responseAction + "] for event [" + responseTokens.get("incidentId") + "]");
    log.info("Processing handleResponseAction with responseTokens: " + responseTokens);

    var incidentId = responseTokens.get("incidentId");
    var responder = responseTokens.get("responder");
    var omwOwner = responseTokens.get("omw_owner");
    var omwPassword = responseTokens.get("omw_password");
    
    if (annotation != null)
    {
        annotation = notePrefix + annotation;
    }

    var commandList = [
          "cmd",
          "/c",
          "cscript.exe",
          "/Nologo",
          OMW_VBS_SCRIPT,
          "-MsgId", incidentId,
          "-Request"
      ];

    switch (String(responseAction).toLowerCase())
    {
        case "acknowledge":
            commandList = commandList.concat([
              "ApOmwAck",
              "-arg1", "\"" + annotation + "\"",
              "-omwuser", "\"" + omwOwner + "\"",
              "-omwpassword", "\"" + omwPassword + "\""
          ]);
          execute(commandList, incidentId, "ApOmwAck");
            break;
        case "own":

            if (responder)
            {
              commandList = commandList.concat([
                  "ApOmwOwn",
                  "-arg1", "\"" + responder + "\"",
                  "-arg2", "\"" + annotation + "\"",
                  "-omwuser", "\"" + omwOwner + "\"",
                  "-omwpassword", "\"" + omwPassword + "\""
              ]);
              execute(commandList, incidentId, "ApOmwOwn");
            }
            else
            {
              log.error("Agent Client: hpomw - Request Script Failed for (" + responseAction + "/" + incidentId + "): No User Specified to Own the Incident.");
            }

            break;

        case "changesevcritical":
            commandList = commandList.concat([
              "ApOmwChgSev",
              "-arg1", "Critical",
              "-arg2", "\"" + annotation + "\"",
              "-omwuser", "\"" + omwOwner + "\"",
              "-omwpassword", "\"" + omwPassword + "\""
          ]);
          execute(commandList, incidentId, "ApOmwChgSev");
            break;

        case "changesevmajor":
            commandList = commandList.concat([
              "ApOmwChgSev",
              "-arg1", "Major",
              "-arg2", "\"" + annotation + "\"",
              "-omwuser", "\"" + omwOwner + "\"",
              "-omwpassword", "\"" + omwPassword + "\""
          ]);
          execute(commandList, incidentId, "ApOmwChgSev");
            break;

        case "changesevminor":
            commandList = commandList.concat([
              "ApOmwChgSev",
              "-arg1", "Minor",
              "-arg2", "\"" + annotation + "\"",
              "-omwuser", "\"" + omwOwner + "\"",
              "-omwpassword", "\"" + omwPassword + "\""
          ]);
          execute(commandList, incidentId, "ApOmwChgSev");
            break;

        case "changesevwarning":
            commandList = commandList.concat([
              "ApOmwChgSev",
              "-arg1", "Warning",
              "-arg2", "\"" + annotation + "\"",
              "-omwuser", "\"" + omwOwner + "\"",
              "-omwpassword", "\"" + omwPassword + "\""
          ]);
          execute(commandList, incidentId, "ApOmwChgSev");
            break;

        case "changesevnormal":
            commandList = commandList.concat([
              "ApOmwChgSev",
              "-arg1", "Normal",
              "-arg2", "\"" + annotation + "\"",
              "-omwuser", "\"" + omwOwner + "\"",
              "-omwpassword", "\"" + omwPassword + "\""
          ]);
          execute(commandList, incidentId, "ApOmwChgSev");
            break;

        case "ignore":
            commandList = commandList.concat([
                "ApOmwAnnotate",
                "-arg1", "\"" + annotation + "\"",
                "-omwuser", "\"" + omwOwner + "\"",
                "-omwpassword", "\"" + omwPassword + "\""
            ]);
            execute(commandList, incidentId, "ApOmwAnnotate");
            break;

        case "annotate":
            commandList = commandList.concat([
                "ApOmwAnnotate",
                "-arg1", "\"" + annotation + "\"",
                "-omwuser", "\"" + omwOwner + "\"",
                "-omwpassword", "\"" + omwPassword + "\""
            ]);
            execute(commandList, incidentId, "ApOmwAnnotate");
            break;
        default:
            throw "Unknown response Action: " + responseAction;
            break;
    }

    log.info("Exit - handleResponseAction");
}

/*
 * checkAPXMLTokens
 *
 * Check that the received response tokens are valid and see if this message needs to be acted on. 
 * Throws an exception if message is invalid.
 * Returns true if the message needs to be processed.
 */
function checkAPXMLTokens(responseTokens)
{
    var result = true;

    switch (String(responseTokens.get("responseEvent")))
    {
        case "RECEIVED_RESPONSE":
        case "STANDALONE_ANNOTATION":
            if(responseTokens.get("responseAction") == null
                || responseTokens.get("responseAction") == ""
                || responseTokens.get("responseAction") == undefined)
                {
                throw "responseAction not set for a RECEIVED_RESPONSE responseEvent.  responseAction required check xMatters scripting to ensure this token is set";
            }
            break;

        case "DEVICE_DELIVERED":
            // Log voice device delivery, but do not annotate to the MS, a callout annotation will be used instead.
            // So just return.
            log.debug(notePrefix + responseTokens.get("responseEvent") + " to " + responseTokens.get("responder"));
            result = false;
            break;

        case "SUCCESSFUL_DELIVERY":
        case "DELIVERY_FAILURE":
            if (!ANNOTATE_DELIVERY)
            {
                log.warn("DELIVERY_ANNOTATIONS are suppressed the following message will not be annotated to IBMNETCOOL event: ["
                    + responseTokens.get("incidentId") + "] " + responseTokens.get("annotation"));
                result = false;
            }
            break;

        default:
            throw "Unknown responseEvent.  Expected: RECEIVED_RESPONSE||SUCCESSFUL_DELIVERY||DELIVERY_FAILURE||DEVICE_DELIVERED. Received: " 
            + responseTokens.get("responseEvent");
            break;
    }
   
    return result;
}

// -----------------------------------------------------------------------------
// Callout annotation processing
//
// If enabled, these result in annotations (notes) being added to the event.
// -----------------------------------------------------------------------------
function handleCalloutAnnotate(incidentId, annotation, apxml)
{

    log.info("Enter - handleCalloutAnnotate - incidentId [" + incidentId + "], annotation [" + annotation + "]");

    log.info( "The following callout annotation is for logging only and will not be annotated to the management system: " + annotation);
    
    log.info("Exit - handleCalloutAnnotate");
}

// -----------------------------------------------------------------------------
// Utility functions
// -----------------------------------------------------------------------------

/**
 * Retrieves the additional tokens from the APXML
 * @param apxml XML message to process
 * @return additional tokens
 * @throws if the additional tokens are missing from the XML.
 */
function getTokens(apxml)
{
    if (apxml.getValue("additionaltokens") != null && !"".equalsIgnoreCase(apxml.getValue("additionaltokens")))
    {
        return xStream.fromXML(apxml.getValue("additionaltokens")); // LinkedHashMap
    }
    throw new Exception("additionaltokens is missing from apxml");
}

/** ***********************************************************************
 *
 * DESCRIPTION
 * This procedure will escape ' and \ characters to prevent conflicts with the
 # SQL command.
 *
 * PARAMETERS
 * sqlChars - The string to escape.
 *
 * RETURNS
 * The escaped sqlChars string.
 *
 ** ***********************************************************************/
function escapeSqlChars( sqlChars )
{
  sqlChars = sqlChars.replace("\\", "\\\\");
  return sqlChars.replace("'", "\\'");
}

/**
 * This procedure will run an external command, providing it the required
 * data tags.
 *
 * GIVEN INPUTS
 * String command - The full path and name of a program to execute, plus
 * any command line arguments.
 *
 * incidentId - The unique ID associated with the event.
 *
 * requestText - The choice selected by the person in response to the event.
 */
function execute(command, incidentId, requestText)
{

  log.info("Agent Client going to execute: " + command );

  // Execute an external command, and wait for it to finish.  By default
  // we append the incident ID so the management system knows what
  // event AlarmPoint is referring to.
  var p = Runtime.getRuntime().exec(command);
  p.waitFor();
 
  //uncomment the code below for debugging to view the output of the command execution
  //var stdInput = new BufferedReader(new InputStreamReader(p.getInputStream()));
  //log.info("***************************OUTPUT " + stdInput.readLine());
  
  log.info("Agent Client: hpomw - Request Script Executed for (" + requestText + "/" + incidentId + "), exit value: " + p.exitValue());
}
